{$A+,B-,D+,E+,F-,I+,L+,N+,O-,R-,S-,V+}
{$M 4000,16384,655350}
uses
  crt,
  graph,
  fonts,
  drivers;

type
  RealType = double;
  CubeType = array[1..16,1..3] of RealType;
  CenterType = record
			  x,y,z : realType;
			end;

CONST
  RADMUL  = 0.017453292519943296; {Pi / 180 : to convert degrees to radians}
  CUBE : CubeType = {array of coordinates for the cube}
    ((30,30,30),(30,-30,30),(-30,-30,30),(-30,30,30),
     (30,30,30),(30,30,-30),(30,-30,-30),(-30,-30,-30),
     (-30,30,-30),(30,30,-30),(30,-30,-30),(30,-30,30),
     (-30,-30,30),(-30,-30,-30),(-30,30,-30),(-30,30,30));

var
  xRot,yRot,zRot : RealType; {Rotatoin values} {Internal use only}
  sinX,sinY,sinZ,cosX,cosY,cosZ : RealType; {Internal use only}

  mx,my,mz : RealType; {Position of object}

  distance : RealType; {distance from eye to object}

  aspect   : RealType; {aspect ratio of screen}

  center   : CenterType; {Point to rotate about}

  xCenter,
  yCenter  : integer;

  viewPort : ViewPortType;

{***********************************************}
{**                                           **}
{**                 Rotate                    **}
{**                                           **}
{***********************************************}
{**                                           **}
{** Sets the rotation angle for projection    **}
{** onto the 3-D plane.                       **}
{**                                           **}
{** Note: x, y, and z are incremental, to     **}
{**       set it, convert degrees to radians  **}
{**       and set rotX, rotY, or rotZ to the  **}
{**       angle of rotation.                  **}
{**                                           **}
{***********************************************}

{x,y,z    = Rotation angle in degrees}

procedure Rotate(x,y,z,xCenter,yCenter,zCenter : RealType);   {Rotates a set of points around 0,0,0}
begin
  center.x := xCenter;
  center.y := yCenter;
  center.z := zCenter;
  xRot := xRot + (x * RADMUL);
  yRot := yRot + (y * RADMUL);
  zRot := zRot + (z * RADMUL);
  sinX := sin(xRot);
  cosX := cos(xRot);
  sinY := sin(yRot);
  cosY := cos(yRot);
  sinZ := sin(zRot);
  cosZ := cos(zRot);
end;

{ xPos,yPos,zPos = Position of viewing          }
{ d              = distance (used for scaling)  }
{ x,y,z          = Viewing angle                }

procedure SetAngle(x,y,z : RealType);  {Sets projection rotation}
var
  xa,ya : Word;
begin
  xRot := x * RADMUL;
  yRot := y * RADMUL;
  zRot := z * RADMUL;
  Rotate(0,0,0,0,0,0);
  GetAspectRatio(xa,ya);
  aspect := xa/ya;
end;

procedure SetViewPos(x,y,z : RealType); {User viewpoint at x,y,z}
begin
  mx := x;
  my := y;
  mz := z;
  xCenter := (viewPort.x2-viewPort.x1) shr 1 - round(mx*(distance/my));
  yCenter := (viewPort.y2-viewPort.y1) shr 1 - round(mz*(distance/my)*aspect);
end;

procedure SetDistance(d : RealType);  {Sets scaling factor}
begin
  distance := d;
  xCenter := (viewPort.x2-viewPort.x1) shr 1 - round(mx*(distance/my));
  yCenter := (viewPort.y2-viewPort.y1) shr 1 - round(mz*(distance/my)*aspect);
end;

{Converts x,y,z to x,y coordinates}
{sx,sy = Coordinates used for plotting on the screen}

procedure Project(x,y,z : RealType;var sx,sy : integer);
{******************************************************}
{** Converts (x,y,z) to (sx,sy)                        }
var
  xa,ya,za : RealType;
begin
  y := -y;
  z := -z;
  x := x - center.x;
  y := y - center.y;
  z := z - center.z;
  xa := (cosX*x)-(sinX*y);
  ya := (sinX*x)+(cosX*y);
  x := (cosY*Xa)+(sinY*z)+center.x+mx;
  za := (cosY*z)-(sinY*Xa);
  y := (cosZ*Ya)-(sinZ*Za)+center.y+my;
  z := (sinZ*Ya)+(cosZ*Za)+center.z+mz;
  sx := round(distance*x/y)+xCenter;
  sy := round(distance*z*aspect/y)+yCenter;
end;

procedure PointP(x,y,z : RealType;color : word);
var
  sx,sy : integer;
begin
  Project(x,y,z,sx,sy);
  PutPixel(sx,sy,color);
end;

procedure Init3D(x1,y1,x2,y2 : word;clip : Boolean);
begin
  SetAngle(0,0,0);
  SetViewPort(x1,y1,x2,y2,clip);
  GetViewSettings(viewPort);
  distance := 400;
  mx := 100;
  my := 200;
  mz := 100;
  xCenter := (viewPort.x2-viewPort.x1) Shr 1 - Round(mx*(distance/my));
  yCenter := (viewPort.y2-viewPort.y1) Shr 1 - Round(mz*(distance/my)*aspect);
end;

procedure Line3D(x1,y1,z1,x2,y2,z2,Cell : RealType;Color : Word);
{***************************************************************}
{** This routine is not used because it is far too slow.        }
{** It draws 3D lines, which can also be done by converting     }
{** both endpoints to 2D and drawing a line between them        }
var
  Dx,Dy,Dz,PlotX,PlotY,PlotZ : RealType;
  Ix,Iy,Iz                   : RealType;
  Xt,Yt,Zt                   : RealType;
  Inc1,Inc2                  : RealType;
  Plot                       : Boolean;
  I1,I2,I3                   : RealType;

  Function Max(x,y : RealType) : RealType;
   begin
    if x > y then Max := x else Max := y;
   end;

  Function Sign(x : RealType) : integer;
   begin
    Sign := 0;
    If x > 0 Then Sign := 1;
    If x < 0 Then Sign := -1;
   end;

begin
  If Cell <= 0.0 Then Cell := 1.0;
  Ix := Abs(x2-x1)/Cell;
  Iy := Abs(y2-y1)/Cell;
  Iz := Abs(z2-z1)/Cell;
  Inc1 := Max(Ix,Iy);
  Inc2 := Max(Ix,Iz);
  PlotX := x1;
  PlotY := y1;
  PlotZ := z1;
  Xt := 0.0;
  Yt := 0.0;
  Zt := 0.0;
  PointP(x1,y1,z1,Color);
  I2 := 0.0;
  Repeat
    Xt := Xt + Ix;
    Yt := Yt + Iy;
    Zt := Zt + Iz;
    Plot := False;
    If Xt > Inc2 Then
     begin
       Plot := True;
       Xt := Xt - Inc2;
       PlotX := PlotX + Sign(Dx);
     end;
    If Yt > Inc1 Then
     begin
       Plot := True;
       Yt := Yt - Inc1;
       PlotY := PlotY + Sign(Dy);
     end;
    If Zt > Inc2 Then
     begin
       Plot := True;
       Zt := Zt - Inc2;
       PlotZ := PlotZ + Sign(Dz);
     end;
    If Plot=True Then PointP(PlotX,PlotY,PlotZ,Color);
    I2 := I2 + Cell;
  Until I2 > Inc2;
end;

procedure Draw3d(x1,y1,z1,x2,y2,z2 : RealType);
{*********************************************}
{** Draws a line from (x1,y1,z1) to (x2,y2,z2)}
var
  sx1,sy1,sx2,sy2 : integer;
begin
  Project(x1,y1,z1,sx1,sy1);
  Project(x2,y2,z2,sx2,sy2);
  Line(sx1,sy1,sx2,sy2);
end;
{*************************************************}

{Draws the cube}

procedure DrawAxis;
var
  x,y : integer;
begin
  SetColor(LightGreen);
  Draw3D(50,0,0,-50,0,0);
  Draw3D(0,-50,0,0,50,0);
  Draw3D(0,0,-50,0,0,50);
  SetColor(White);
  Project(50,0,0,x,y);
  OutTextXY(x,y,'x');
  Project(0,50,0,x,y);
  OutTextXY(x,y,'y');
  Project(0,0,50,x,y);
  OutTextXY(x,y,'z');
  Project(-50,0,0,x,y);
  OutTextXY(x,y,'-x');
  Project(0,-50,0,x,y);
  OutTextXY(x,y,'-y');
  Project(0,0,-50,x,y);
  OutTextXY(x,y,'-z');
  SetColor(LightRed);
end;

procedure DrawCUBE;

var
  t, t1, x, y, i : integer;
  page : integer;
begin
  page := 0;
  for t := 1 to 90 do
   begin
	if keyPressed then exit;
     SetActivePage(page);
	page := page xor 1;
     SetVisualPage(page);
     ClearViewPort;
     DrawAxis;
     Project(CUBE[1,1],CUBE[1,2],CUBE[1,3],x,y);
     SetColor(LightRed);
     MoveTo(x,y); {Set starting point}
	For t1 := 2 to 16 do
      begin
	   Project(CUBE[t1,1],CUBE[t1,2],CUBE[t1,3],x,y);
        LineTo(x,y);
      end;
	Rotate(2,0,0,0,0,0);
   end;

  for t := 1 to 90 do
   begin
     if KeyPressed then Exit;
     SetActivePage(page);
     page := page xor 1;
     SetVisualPage(page);
     ClearViewPort;
     DrawAxis;
     Project(CUBE[1,1],CUBE[1,2],CUBE[1,3],x,y);
     SetColor(LightRed);
     MoveTo(x,y);
	for t1 := 2 to 16 do
      begin
	   Project(CUBE[t1,1],CUBE[t1,2],CUBE[t1,3],x,y);
        LineTo(x,y);
      end;
	Rotate(0,2,0,0,0,0);
   end;

  for t := 1 to 90 do
   begin
     if KeyPressed then Exit;
     SetActivePage(page);
     page := page xor 1;
     SetVisualPage(page);
     ClearViewPort;
     DrawAxis;
     Project(CUBE[1,1],CUBE[1,2],CUBE[1,3],x,y);
     SetColor(LightRed);
     MoveTo(x,y);
	For t1 := 2 to 16 do
      begin
	   Project(CUBE[t1,1],CUBE[t1,2],CUBE[t1,3],x,y);
        LineTo(x,y);
      end;
	Rotate(0,0,2,0,0,0);
   end;
  xRot := 0.0;
  yRot := 0.0;
  zRot := 0.0;
end;

var
  GraphDriver,GraphMode : integer;
  xAspect,yAspect : word;
  c : char;

begin
  DirectVideo := False;
  GraphDriver := Ega;
  GraphMode := EgaHi;
  if RegisterBGIdriver(@EGAVGADriverProc) < 0 then
   begin
     WriteLn('EGAVGA.BGI not properly linked!');
     Halt(1);
   end;
  if RegisterBGIfont(@GothicFontProc) < 0 then
   begin
     WriteLn('Goth.CHR not properly linked!');
     Halt(1);
   end;
  InitGraph(GraphDriver,GraphMode,'');
  SetTextStyle(GothicFont,0,5);
  SetActivePage(0);
  SetVisualPage(1);
  SetFillStyle(SolidFill,1);
  FloodFill(1,1,1);

  OutTextXY(80,0,'Press any key to continue');

  SetVisualPage(0);
  SetActivePage(1);
  FloodFill(1,1,1);

  OutTextXY(80,0,'Press any key to continue');

  SetActivePage(0);
  SetVisualPage(1);

  SetTextStyle(SmallFont,0,5);

  Init3D(160,70,480,300,True);
  SetViewPos(100,200,100);
  SetDistance(400);

  repeat
    DrawCube;
  until KeyPressed;

  SetViewPort(0,0,GetMaxX,GetMaxY,False);
  c := ReadKey;
  TextMode(Co80);
end.
