UNIT KEYSTUFF;

INTERFACE

PROCEDURE StuffKey(KCh:Char; KScan:Byte);

IMPLEMENTATION

{============================================================================}
{ This routine stuffs one keystroke in the keystroke buffer.  The result is  }
{ equivalent to the user hitting the key.  For a normal keystroke the usage  }
{ is to call the procedure with KCh equal to the desired character and KScan }
{ equal to #0 like so:                                                       }
{                          StuffKey('Z',0);                                  }
{ This stuffs the buffer with Z                                              }

{ If you want to place a function key in the buffer then set the KCh to #0   }
{ and put the scan code for the function key in KScan like so:               }
{                          StuffKey(#0,77);                                  }
{ This stuffs the buffer with a right arrow key                              }

PROCEDURE StuffKey(KCh:Char; KScan:Byte);

TYPE
  KeySeq = Record                { Keystroke record }
             KeyCh : Char;
             ScanCode : Byte;
           End;

  KeyBuf = Record                { Memory map of Keyboard buffer }
     Head : Integer;
     Tail : Integer;
     Keys : Array[1..16] of KeySeq;
   End;

VAR
     NextPos	: integer;
     Index	: integer;
     KeyBuffer  : KeyBuf absolute $0040:$001A;   { Memory location of buffer }

BEGIN
  With KeyBuffer do
     Begin
       NextPos := Tail + 2;          { find the next character position }
       if NextPos  > Ofs(Keys[16]) then     { wrap around }
          NextPos := Ofs(Keys[1]);
       Index := ((Tail - $1E) div 2) + 1;   { Compute buffer index }
       Inline($FA);			    { Disable interrupts }
       Keys[Index].KeyCh := KCh;            { Put character in buffer }
       Keys[Index].ScanCode := KScan;       { Put scan code in buffer }
       Tail := NextPos;                     { Set Pointer }
       Inline($FB);			    { Enable interrupts }
     End;

END;


END. {of unit}
