{======================================================================}
UNIT LOADFILE;

INTERFACE
USES ADDRIO,CRT;
PROCEDURE Load_ASCII (AFile:String);
PROCEDURE Load_MEM   (MFile:String);
PROCEDURE Load_PAK   (PFile:String);
PROCEDURE Load_WIN   (WFile:String);

IMPLEMENTATION

VAR
   LoadScr : Screen;
   TempStr : String;
   XX1,YY1,XX2,YY2 : Integer;
   XW1        : Map Absolute XX1;    { This trick allows the first }
   YW1        : Map Absolute YY1;    { four records of a window file }
   XW2        : Map Absolute XX2;    { to actually be integers defining }
   YW2        : Map Absolute YY2;    { the window coordinates }

{======================================================================}
PROCEDURE Load_ASCII(AFile : String);
{                                                     }
{ This routine loads an ASCII format file as created  }
{ by BOX onto the screen.                             }
{                                                     }

VAR
    FilevarA   : Text;
    II         : Integer;

BEGIN
      Assign(FilevarA,AFile);
      {$I-} Reset(FilevarA); {$I+}
      If IOresult = 0 then                        {found good file name}
       Begin
        ClrScr;
        For II := 1 to 24 do
           Begin
             Readln(FilevarA,TempStr);
             WriteSt(TempStr,1,II);
           End;
        { Handle special case of last character on 25th line so screen }
        { won't scroll up }
        Readln(FilevarA,TempStr);
        If(TempStr[0] = Chr(80)) then
             Begin
               TempStr[0] := Chr(79);
               WriteSt(Tempstr,1,25);
               WriteCh(TempStr[80],80,25)
             End
        Else WriteSt(TempStr,1,25);
        Close(FilevarA);
       End

      Else                                        {couldn't find file  }
        Begin
          GoToXY(1,24);
          Write('ERROR - Could not find file ',AFile);
        End;

END; {Load_ASCII}

{======================================================================}
PROCEDURE Load_MEM(MFile : String);
{                                                     }
{ This routine loads a memory format file as created  }
{ by BOX directly into the video buffer.  Since a     }
{ memory format file is the same shape as the video   }
{ buffer, all that needs to be done is to move the    }
{ screen into the buffer with MoveToScreen in IOSTUFF.}
{                                                     }

VAR
    FilevarM   : File;

BEGIN
      Assign(FilevarM,MFile);
      {$I-} Reset(FilevarM,4000); {$I+}
      If IOresult = 0 then                        {found good file name}
        Begin
          BlockRead(FilevarM,LoadScr,1);
          MoveToScreen(LoadScr,Video^,4000);
          Close(FilevarM);
        End
      Else                                        {couldn't find file  }
        Begin
          GoToXY(1,24);
          Write('ERROR - Could not find file ',MFile);
        End;
END; {Load_MEM}

{======================================================================}
PROCEDURE Load_PAK(PFile:String);
{                                                     }
{ This procedure loads a Packed Format screen created }
{ by BOX.  The Packed format utilizes a run-length    }
{ encoding scheme that must be unpacked.  Each record }
{ in a Packed Format file is three bytes long. Byte 1 }
{ is the run length, i.e. the number of characters to }
{ repeat.  Byte 2 is the character to repeat and      }
{ byte 3 is the attribute of the character.           }
{                                                     }
TYPE
   Pack        = Record
                  PackNm : Byte;  {run length}
                  PackCh : Char;  {repeated character}
                  PackAt : Byte;  {repeated attribute}
                 End;

VAR
    FilevarM   : File;
    Packbuf    : Array[1..2000] of Pack;
    II,JJ,Sloc,SX,SY,NumRec  : Integer;

BEGIN
   Sloc := 1;              {SLoc is location on screen}
   Assign(FilevarM,PFile);
   {$I-} Reset(FilevarM); {$I+}
   If IOresult = 0 then          {found good file name}
     Begin
        BlockRead(FilevarM,PackBuf,48,NumRec);
        JJ := 0;
        While Sloc < 2001 do
        Begin
          JJ := JJ + 1;
          For II := 1 to Packbuf[JJ].PackNm do
           Begin
            SY := (SLoc-1) div 80 + 1;       {row}
            SX := (SLoc-1) mod 80 + 1;       {column}
            LoadScr[SY,SX].ScrCh := Packbuf[JJ].PackCh;
            LoadScr[SY,SX].ScrAt := Packbuf[JJ].PackAt;
            SLoc := SLoc + 1;
           End;
        End;
        MoveToScreen(LoadScr,Video^,4000);
        Close(FilevarM);
   End
   Else                            {couldn't find file}
     Begin
       GoToXY(1,24);
       Write('ERROR - Could not find file ',PFile);
     End;
END;

{======================================================================}
PROCEDURE Load_WIN   (WFile:String);
{                                                     }
{ This routine loads a window format file as created  }
{ by BOX on the screen.  The first four records in    }
{ the file define the location of the window and the  }
{ following records contain character/attribute pairs }
{ which define the contents of the window.            }
{                                                     }

VAR
    FilevarW   : File of Map;
    II,JJ      : Integer;
BEGIN
      Assign(FilevarW,WFile);
      {$I-} Reset(FilevarW); {$I+}
      If IOresult = 0 then                      {found good file name}
        Begin
          MoveFromScreen(Video^,LoadScr,4000);  { Save the current screen }
                                                { to a holding area }

          Read(FilevarW,XW1);     { Read in the coordinates of the window }
          Read(FilevarW,YW1);
          Read(FilevarW,XW2);
          Read(FilevarW,YW2);
                                                { Loop on coordinates }
          For II := YY1 to YY2 do
            For JJ := XX1 to XX2 do
               Read(FilevarW,LoadScr[II,JJ]);   { Read one record and put }
                                                { that character in screen }
                                                { holding area }
          MoveToScreen(LoadScr,Video^,4000);    { When done restore the screen }
          Close(FilevarW);
        End
      Else                                      { couldn't find file  }
        Begin
          GoToXY(1,24);
          Write('ERROR - Could not find file ',WFile);
        End;
END; {Load_WIN}

END.

